----------------------------------------------------------------------------------------------------------
------------------------------------------------MageGate------------------------------------------------
----------------------------------------------------------------------------------------------------------
-- Written by: Endar_Ren																				--
----------------------------------------------------------------------------------------------------------
----------------------------------------------Change History----------------------------------------------
----------------------------------------------------------------------------------------------------------
-- Version 1.1 Changes 8/25/2010																		--
-- *  Fixed use of addon when a target or a group member is casting a portal.							--
-- *  Corrected GateGroup function so that the Party channel is used outside of an instance.			--
-- *  Corrected spelling error:  "Dailing".																--
--======================================================================================================--							
-- Version 1.2 Changes 10/5/10																			--
-- *  Fixed the nothing happening in Party group type. (I think)										--
--======================================================================================================--
-- Version 1.3 Changes 10/13/2010																		--
-- *  Now usable in 4.0.1.																				--
--======================================================================================================--
-- Version 1.4 Changes 11/3/2010																		--
-- *  Add in Slash Command to turn the addon on and off. (Does not disable or enable the addon)			--
--======================================================================================================--
-- Version 1.4.1 Changes 11/18/2010																		--
-- *  Added Confirm Summons event and "Incoming traveller" message.  Need to test.	FAILED				--
-- *  Changed GateGroup to use number in the party to see if you are alone.	Need to test.  FAILED		--
-- *  Added version number field.																		--
--======================================================================================================--
-- Version 1.4.2  Changed 1/3/2011																		--
-- * Adding in sound effects.  Sounds given to me by Ricardo58.  Thank you.								--
--======================================================================================================--
-- Version 1.4.3 Changed 4/26/2011																		--
-- * TOC version number changed to 40100.																--
--======================================================================================================--
-- Version 2 Changed 7/1/2011																			--
-- * TOC version number changed to 40200.																--
-- * Ace3 Library implemented.																			--
-- * New Slash commands:  sfx & autosum.																--
-- * New Sound Effect for summon																		--
-- * Stargate Atlantis Dialing abort sound effect option added.											--
--======================================================================================================--
-- Version 2.1 Changed 3/5/2012																			--
-- * TOC AUTOSUMM saved variable renamed to AUTOSUMM2.													--
-- * AUTOSUMM2 is set to false by default now.															--
-- * Fixed get for abortfx to actually return ABORTSTY.													-- 
--======================================================================================================--
-- Version 3.0 Changed 5/2/2012																			--
-- * Changes made by Daimanu.																			--
-- * Portal identification method changed to deal with other languages.									--
-- * Chevron count up added, with customizable chevron number range.									--
-- *  French translation added.																			--
--======================================================================================================--
MageGate = LibStub("AceAddon-3.0"):NewAddon("MageGate","AceConsole-3.0","AceComm-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MageGate")

local AceConfig = LibStub("AceConfig-3.0")
local MG_VERSION = 20000;
channeling =false;
NeooptionTable = {
		name = "MageGate",
		handler = MageGate,
		type = 'group',
		args = {

				sfx =   {
									name="Sound Effects",
									desc = L["SOUND_FX_DESC"],
									type = "execute",
									func ="SoundEffects"
						},
				autosum  =   {
									name="Auto Summon",
									desc = L["AUTO_SUMMON_DESC"],
									type = "execute",
									func ="AutoSummon"
						},
				abortfx  =   {
									name="Abort Effect",
									desc = string.format(L["ABORT_FX_DESC"], "sg1", "sga"),
									type = "toggle",
									set ="ChangeAbortSFX",
									get=function () return ABORTSTY end
						},
				chevrons  =  {
									name = "Number of chevrons",
									desc = L["CHEVRONS_COUNT_DESC"],
									type = "range",
									min  = 7,
									max  = 9,
									step = 1,
									set  = "SetChevronCount",
									get  = function () return CHEVRONCOUNT end
						}
				
				}
			}

LibStub("AceConfig-3.0"):RegisterOptionsTable("MageGate", NeooptionTable, {"mgate"})

--Sorted list
local portalsID = {
	10059, -- StormWind
	11416, -- IronForge
	11417, -- Orgrimmar
	11418, -- Undercity
	11419, -- Darnassus
	11420, -- Thunder Bluff
	32266, -- Exodar
	32267, -- SilverMoon
	33691, -- Shattrath (Alliance)
	35717, -- Shattrath (Horde)
	49360, -- Theramore
	49361, -- Stonard
	53142, -- Dalaran
	88345, -- Tol Barad (Alliance)
	88346  -- Tol Barad (Horde)
}

--- Indicates if the given spell is a portal spell
-- @param spellID
function isAPortal( spellID )
	for index, oneId in pairs( portalsID ) do
		if oneId == spellID then
			return true
		end
		if oneId > spellID then
			return false
		end
	end
	return false
end

--- Timer functions
local pTimer = nil
local nTicCount = 1
--- Handler for a timer tic
function MageGate:ChevronNLocked()
	if channeling == false then
		self:CancelChevrons()
		return
	end
	if (nTicCount >= CHEVRONCOUNT) then
		self:CancelChevrons()
		return
	end
	SendChatMessage(string.format(L["CHEVRON_N_LOCKED"], nTicCount), self:GateGroup())
	nTicCount = nTicCount + 1
end
--- Called when a portal is cancelled
function MageGate:CancelChevrons()
	MageGate:CancelTimer(pTimer, false)
	pTimer = nil
	channeling = false
	nTicCount = 1
end

function MageGate:OnInitialize()
	if GATEON == nil then
		GATEON = true;
	end
	if SOUNDFX == nil then
		SOUNDFX = true
	end
	if AUTOSUMM2 == nil then
		AUTOSUMM2 = false;
	end
	if ABORTSTY == nil then
		ABORTSTY = "sg1"
	end
	if CHEVRONCOUNT == nil then
		CHEVRONCOUNT = 7
	end
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED_QUIET")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED") 
	self:RegisterEvent("CONFIRM_SUMMON")
end


function MageGate:OnEnabled()

	
end

function MageGate:OnDisable()

end

function MageGate:AutoSummon(info, val)
	AUTOSUMM2 = AUTOSUMM2 == false;
	if AUTOSUMM2 then
		print(L["AUTO_SUMMON_ON"])
	else
		print(L["AUTO_SUMMON_OFF"])
	end
end
function MageGate:ChangeAbortSFX(info, val)
	if string.len(val) == 3 then
		if string.find("sg1",val) == nil then
			ABORTSTY = "sga"
		
		else
			ABORTSTY = "sg1"
		end
	else
		print(string.format(L["ABORT_FX_INVALID"], "sg1", "sga"))
	end

end
function MageGate:SetChevronCount(info, val)
	local min, max = NeooptionTable.args.chevrons["min"], NeooptionTable.args.chevrons["max"] -- Better way to get params ?
	if (val >= min) and (val <= max) then
		CHEVRONCOUNT = val
	else
		print(string.format(L["CHEVRONS_COUNT_INVALID"], min, max))
	end
end
function MageGate:UNIT_SPELLCAST_START(eveName, unitID, spell, rank, lineID, spellID)
--print("start 1")
	local beg, last = string.find(spell, L["PORTAL_SEP"]);
--print(string.sub(spell, last+2))
	if isAPortal(spellID) and (unitID == "player") then
		channeling = true;
		SendChatMessage(string.format(L["DIALING_START"], string.sub(spell, last+2)), self:GateGroup()); --Displays portal destination

	-- Timer
		local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spell)
		pTimer = MageGate:ScheduleRepeatingTimer("ChevronNLocked", castTime / (CHEVRONCOUNT*1000) )
	-- ! Timer
	end
end
function MageGate:UNIT_SPELLCAST_STOP(eveName, unitID, spell, rank, lineID, spellID)
--print ("pots")
end
function MageGate:UNIT_SPELLCAST_SUCCEEDED(eveName, unitID, spell, rank, lineID, spellID)
--print ("succ")
	--Doesn't cancel timer, it might have stoped itself
	if isAPortal(spellID) and (unitID == "player") then
		channeling = false;
		if SOUNDFX then
			PlaySoundFile("Interface\\AddOns\\MageGate\\Sound Files\\Kawoosh 1.ogg");
		end
		SendChatMessage(string.format(L["CHEVRON_N_ENGAGED"], CHEVRONCOUNT), self:GateGroup());
	end
end
function MageGate:UNIT_SPELLCAST_INTERRUPTED(eveName, unitID, spell, rank, lineID, spellID)
	if channeling == false then
	return
	end
	if isAPortal(spellID) and (unitID == "player") then
		channeling = false;
		SendChatMessage(L["DIALING_ABORTED"], self:GateGroup());
		self:CancelChevrons();
		if SOUNDFX then
			if string.find("sg1", ABORTSTY) ~= nil  then 
				PlaySoundFile("Interface\\AddOns\\MageGate\\Sound Files\\SG-1 Dial Abort.ogg")
			else
			--Atlantis Dial Abort
				PlaySoundFile("Interface\\AddOns\\MageGate\\Sound Files\\Atlantis Dial Abort.ogg")
			end
		end
	end
end
function MageGate:UNIT_SPELLCAST_FAILED_QUIET(eveName, unitID, spell, rank, lineID, spellID)
	if channeling == false then
		return
	end
	if isAPortal(spellID) and (unitID == "player") then
		channeling = false;
		SendChatMessage(L["DIALING_ABORTED"], self:GateGroup());
		self:CancelChevrons();
		if SOUNDFX then
			if string.find("sg1", ABORTSTY) ~= nil  then 
				PlaySoundFile("Interface\\AddOns\\MageGate\\Sound Files\\SG-1 Dial Abort.ogg")
			else
			--Atlantis Dial Abort
				PlaySoundFile("Interface\\AddOns\\MageGate\\Sound Files\\Atlantis Dial Abort.ogg")
			end
		end
	end
end
function MageGate:CONFIRM_SUMMON(eveName, ...)
	--Special thanks to Garshaw of Mok`Nathal for this idea.
	if AUTOSUMM2 == true then
		local chann = self.GateGroup();
		SendChatMessage(L["CONFIRM_SUMMON"], chann);
		ConfirmSummon()
		if SOUNDFX == true then
			PlaySoundFile("Interface\\AddOns\\MageGate\\Sound Files\\Rings 1.ogg")
		end

	end
end

function MageGate:SoundEffects(info,val)
	SOUNDFX = SOUNDFX == false;
	if SOUNDFX then
		print(L["SOUND_FX_ON"])
	else
		print(L["SOUND_FX_OFF"])
	end
end

--This function is used to determine which kind of group,
--if any, the player is in.  It returns the channel name
--needed to reach all the players in that group.
--The return value is a String.

function MageGate:GateGroup ()
	if GetNumPartyMembers()>0 then
		if (UnitInParty("player") ~= nil) then
			if (UnitInRaid("player") ~= nil) then
				return "RAID"
			end
			return "PARTY"
		end
	else
		if (UnitInRaid("player") ~= nil) then
				return "RAID"
		else
			return "SAY"
		end
	end
end